#include <stdlib.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qtabwidget.h>
#include <qframe.h>
#include <qvgroupbox.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qheader.h>
#include <qwhatsthis.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qbuttongroup.h>

#include <kkeydialog.h>
#include <kglobal.h>
#include <kconfig.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kapplication.h>

#include "rules.h"
#include "kcmlayout.h"
#include "kcmlayout.moc"
#include "pixmap.h"
#include "kcmmisc.h"
#include "kcmlayoutwidget.h"

#include <X11/Xlib.h>


static const char* switchModes[] = {
  "Global", "WinClass", "Window"
};

static QString lookupLocalized(const QDict<char> &dict, const QString& text)
{
  QDictIterator<char> it(dict);
  while (it.current())
    {
      if ( i18n(it.current()) == text )
        return it.currentKey();
      ++it;
    }

  return QString::null;
}

OptionListItem::OptionListItem( OptionListItem *parent, const QString &text,
    Type tt, const QString &optionName )
    : QCheckListItem( parent, text, tt ), m_OptionName( optionName )
{
}

OptionListItem::OptionListItem( QListView *parent, const QString &text,
    Type tt, const QString &optionName )
    : QCheckListItem( parent, text, tt ), m_OptionName( optionName )
{
}

OptionListItem * OptionListItem::findChildItem( const QString& optionName )
{
  OptionListItem *child = static_cast<OptionListItem *>( firstChild() );

  while ( child )
    {
      if ( child->optionName() == optionName )
        break;
      child = static_cast<OptionListItem *>( child->nextSibling() );
    }

  return child;
}

LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
  : KCModule(parent, name), rules(0)
{
  QVBoxLayout *main = new QVBoxLayout(this, 0, KDialog::spacingHint());

  widget = new LayoutConfigWidget(this, "widget");
  main->addWidget(widget);

  connect( widget->chkEnable, SIGNAL( toggled( bool )), this, SLOT(changed()));
  connect( widget->chkShowSingle, SIGNAL( toggled( bool )), this, SLOT(changed()));
  connect( widget->comboModel, SIGNAL(activated(int)), this, SLOT(changed()));

  connect( widget->btnAdd, SIGNAL(clicked()), this, SLOT(changed()));
  connect( widget->btnAdd, SIGNAL(clicked()), this, SLOT(add()));
  connect( widget->btnRemove, SIGNAL(clicked()), this, SLOT(changed()));
  connect( widget->btnRemove, SIGNAL(clicked()), this, SLOT(remove()));

  connect( widget->comboVariant, SIGNAL(activated(int)), this, SLOT(changed()));
  connect( widget->comboVariant, SIGNAL(activated(int)), this, SLOT(variantChanged()));
  connect( widget->listLayoutsDst, SIGNAL(selectionChanged(QListViewItem *)), 
		this, SLOT(layoutSelChanged(QListViewItem *)));

  connect( widget->btnUp, SIGNAL(clicked()), this, SLOT(changed()));
  connect( widget->btnUp, SIGNAL(clicked()), this, SLOT(moveUp()));
  connect( widget->btnDown, SIGNAL(clicked()), this, SLOT(changed()));
  connect( widget->btnDown, SIGNAL(clicked()), this, SLOT(moveDown()));

//  connect( widget->chkUSGroup, SIGNAL(clicked()), this, SLOT(changed()));

  connect( widget->grpSwitching, SIGNAL( clicked( int ) ), SLOT(changed()));

  widget->listLayoutsSrc->setColumnWidth(0, 28);
  widget->listLayoutsDst->setColumnWidth(0, 28);
  widget->listLayoutsDst->setSorting(-1);
#if 0
  widget->listLayoutsDst->setResizeMode(QListView::LastColumn);
  widget->listLayoutsSrc->setResizeMode(QListView::LastColumn);
#endif

  //Read rules - we _must_ read _before_ creating xkb-options comboboxes
  ruleChanged("xfree86");

  makeOptionsTab();

  load();
}


LayoutConfig::~LayoutConfig()
{
  delete rules;
}

void LayoutConfig::add()
{
    QListViewItem* sel = widget->listLayoutsSrc->selectedItem();
    if( sel == 0 )
	return;
	
    widget->listLayoutsDst->insertItem(sel);
    sel->moveItem(widget->listLayoutsDst->lastItem());
    widget->listLayoutsSrc->setSelected(sel,true);
    layoutSelChanged(sel);
}

void LayoutConfig::remove()
{
    QListViewItem* sel = widget->listLayoutsDst->selectedItem();
    QListViewItem* newSel = 0;
    
    if( sel == 0 )
	return;
	
    if( sel->itemBelow() )
	newSel = sel->itemBelow();
    else
    if( sel->itemAbove() )
	newSel = sel->itemAbove();
	
    widget->listLayoutsSrc->insertItem(sel);
    if( newSel )
	widget->listLayoutsSrc->setSelected(newSel,true);
    layoutSelChanged(newSel);
}

void LayoutConfig::moveUp()
{
    QListViewItem* sel = widget->listLayoutsDst->selectedItem();
    if( sel == 0 || sel->itemAbove() == 0 )
	return;
	
    if( sel->itemAbove()->itemAbove() == 0 ) {
	widget->listLayoutsDst->takeItem(sel);
	widget->listLayoutsDst->insertItem(sel);
	widget->listLayoutsDst->setSelected(sel, true);
    }
    else
	sel->moveItem(sel->itemAbove()->itemAbove());
}

void LayoutConfig::moveDown()
{
    QListViewItem* sel = widget->listLayoutsDst->selectedItem();
    if( sel == 0 || sel->itemBelow() == 0 )
	return;
	
    sel->moveItem(sel->itemBelow());
}


void LayoutConfig::variantChanged()
{
    QListViewItem* selLayout = widget->listLayoutsDst->selectedItem();
    if( !selLayout ) {
      widget->comboVariant->clear();
      widget->comboVariant->setEnabled(false);
      return;
    }
    
    QString kbdLayout = lookupLocalized( rules->layouts(), selLayout->text(1) );
    variants.replace(kbdLayout, widget->comboVariant->currentText().latin1());
}

void LayoutConfig::layoutSelChanged(QListViewItem *sel)
{

    widget->comboVariant->clear();
    widget->comboVariant->setEnabled( sel != 0 );

    if( sel == 0 ) {
	updateLayoutCommand();
	return;
    }

    QString kbdLayout = lookupLocalized( rules->layouts(), sel->text(1) );
    QStringList vars = rules->getVariants(kbdLayout);

    if( vars.count() == 0 ) {	// cowardly running away
	updateLayoutCommand();
	return;
    }

    char* variant = variants[kbdLayout];
    widget->comboVariant->insertStringList(vars);

    if( variant ) {
	widget->comboVariant->setCurrentText(variant);
    }
    else {
      widget->comboVariant->setCurrentItem(0);
      variants.insert(kbdLayout, widget->comboVariant->currentText().latin1());
    }
    
    updateLayoutCommand();
}

QWidget* LayoutConfig::makeOptionsTab()
{
  QListView *listView = widget->listOptions;

  listView->setMinimumHeight(150);
  listView->setSortColumn( -1 );
  listView->setColumnText( 0, i18n( "Options" ) );
  listView->clear();

  connect(listView, SIGNAL(clicked(QListViewItem *)), SLOT(changed()));
  connect(listView, SIGNAL(clicked(QListViewItem *)), SLOT(updateOptionsCommand()));

  connect(widget->checkResetOld, SIGNAL(toggled(bool)), SLOT(changed()));
  connect(widget->checkResetOld, SIGNAL(toggled(bool)), SLOT(updateOptionsCommand()));

  //Create controllers for all options
  QDictIterator<char> it(rules->options());
  OptionListItem *parent;
  for (; it.current(); ++it)
  {
    if (!it.currentKey().contains(':'))
    {
      if( it.currentKey() == "ctrl" || it.currentKey() == "caps"
          || it.currentKey() == "altwin" ) {
        parent = new OptionListItem(listView, it.current(),
            QCheckListItem::RadioButtonController, it.currentKey());
        OptionListItem *item = new OptionListItem(parent, i18n( "None" ),
            QCheckListItem::RadioButton, "none");
        item->setState(QCheckListItem::On);
      }
      else {
        parent = new OptionListItem(listView, i18n( it.current() ),
            QCheckListItem::CheckBoxController, it.currentKey());
      }
      parent->setOpen(true);
      optionGroups.insert(i18n(it.currentKey().local8Bit()), parent);
    }
  }

  it.toFirst();
  for( ; it.current(); ++it)
  {
    QString key = it.currentKey();
    int pos = key.find(':');
    if (pos >= 0)
    {
      OptionListItem *parent = optionGroups[key.left(pos)];
      if (parent != NULL) {
      // workaroung for mistake in rules file for xkb options in XFree 4.2.0
        QString text(it.current());
        text = text.replace( "Cap$", "Caps." );
        if( parent->type() == QCheckListItem::RadioButtonController )
            new OptionListItem(parent, i18n(text.latin1()),
                QCheckListItem::RadioButton, key);
        else
            new OptionListItem(parent, i18n(text.latin1()),
                QCheckListItem::CheckBox, key);
      }
    }
  }

  //scroll->setMinimumSize(450, 330);

  return listView;
}

void LayoutConfig::updateOptionsCommand()
{
  QString setxkbmap;
  QString options = createOptionString();
  
  if( !options.isEmpty() ) {
    setxkbmap = "setxkbmap -option "; //-rules " + m_rule
    if( widget->checkResetOld->isChecked() )
      setxkbmap += "-option ";
    setxkbmap += options;
  }
  widget->editCmdLineOpt->setText(setxkbmap);
}

void LayoutConfig::updateLayoutCommand()
{
  QString setxkbmap;
  QListViewItem* sel = widget->listLayoutsDst->selectedItem();

  if( sel ) {
    QString kbdLayout = lookupLocalized(rules->layouts(), sel->text(1));
//    char* variant = variants[kbdLayout];
    QString variant = widget->comboVariant->currentText();

    setxkbmap = "setxkbmap"; //-rules " + m_rule
    setxkbmap += " -model " + lookupLocalized(rules->models(), widget->comboModel->currentText())
      + " -layout ";
//    if( widget->chkUSGroup->isChecked() )
//      setxkbmap += "us,";
    setxkbmap += kbdLayout;

    if( !variant.isEmpty() ) {
      setxkbmap += " -variant ";
//      if( widget->chkUSGroup->isChecked() )
//        setxkbmap += ",";
      setxkbmap += variant;
    }
  }
  widget->editCmdLine->setText(setxkbmap);
}

void LayoutConfig::changed()
{
  updateLayoutCommand();
  emit KCModule::changed(true);
}


void LayoutConfig::load()
{
  // open the config file
  KConfig *config = new KConfig("kxkbrc", true);
  config->setGroup("Layout");

  bool use = config->readBoolEntry( "Use", false );

  // find out which rule applies
  QString rule = config->readEntry("Rule", "xfree86");

  // update other files
  ruleChanged(rule);

  // find out about the model
  QString model = config->readEntry("Model", "pc104");

  // TODO: When no model and no layout are known (1st start), we really
  // should ask the X-server about it's settings!

  QString m_name = rules->models()[model];
  widget->comboModel->setCurrentText(i18n(m_name.local8Bit()));

  QString layout = config->readEntry("Layout", "us");
  QString l_name = rules->layouts()[layout];

  QStringList otherLayouts = config->readListEntry("Additional");
  if( !l_name.isEmpty() ) {
    otherLayouts.prepend(layout);


  }
// to optimize we should have gone from it.end to it.begin
  for ( QStringList::Iterator it = otherLayouts.begin(); it != otherLayouts.end(); ++it ) {
    QListViewItemIterator src_it( widget->listLayoutsSrc );
    for ( ; src_it.current(); ++src_it ) {
	QListViewItem* srcItem = src_it.current();
	if ( *it == lookupLocalized(rules->layouts(), src_it.current()->text(1)) ) {
    	    widget->listLayoutsDst->insertItem(srcItem);
	    srcItem->moveItem(widget->listLayoutsDst->lastItem());
	    break;	
	}
    }
  }

// reading variants
  QStringList vars = config->readListEntry("Variants");
  rules->parseVariants(vars, variants);

  bool showSingle = config->readBoolEntry( "ShowSingle", false );
  widget->chkShowSingle->setChecked(showSingle);


  bool resetOld = config->readBoolEntry("ResetOldOptions", false);
  widget->checkResetOld->setChecked(resetOld);
  QStringList options = config->readListEntry("Options");

  for (QStringList::Iterator it = options.begin(); it != options.end(); ++it)
    {
      QString option = *it;
      QString optionKey = option.mid(0, option.find(':'));
      QString optionName = rules->options()[option];
      OptionListItem *item = optionGroups[optionKey];
      if (item != NULL) {
        OptionListItem *child = item->findChildItem( option );

        if ( child )
          child->setState( QCheckListItem::On );
	    else
	      kdDebug() << "load: Unknown option " << option << endl;
      }
      else {
	    kdDebug() << "load: Unknown option group " << optionKey << endl;
      }
    }

  QString swMode = config->readEntry("SwitchMode", "Global");
  widget->grpSwitching->setButton(0);

   for(int ii=1; ii<3; ii++)
    if( swMode == switchModes[ii] )
	widget->grpSwitching->setButton(ii);

  delete config;

  widget->chkEnable->setChecked( use );
  widget->grpLayouts->setEnabled(use);
  widget->grpSwitching->setEnabled(use);
  
  updateOptionsCommand();
}

void LayoutConfig::ruleChanged(const QString &rule)
{
  if( rule == m_rule )
    return;

  m_rule = rule;

  QString model; //, layout;
  if (rules)
    {
      model = lookupLocalized(rules->models(), widget->comboModel->currentText());
//      layout = lookupLocalized(rules->layouts(), layoutCombo->currentText());
    }

  delete rules;
  rules = new KeyRules(rule);

  QStringList tmp;
  widget->comboModel->clear();
  QDictIterator<char> it(rules->models());
  while (it.current())
    {
      tmp.append(i18n(it.current()));
      ++it;
    }
  tmp.sort();
  widget->comboModel->insertStringList(tmp);


  // fill in the additional layouts  -- moved from load() by A. Rysin
  widget->listLayoutsSrc->clear();
  QDictIterator<char> it2(rules->layouts());
  while (it2.current())
    {
      QCheckListItem *item = new QCheckListItem(widget->listLayoutsSrc, "");
      QString addLayout = it2.currentKey();
      item->setPixmap(0, findPixmap(addLayout));
      item->setText(1, i18n(it2.current()) );
      item->setText(2, "(" + addLayout + ")" );
      ++it2;
    }
  widget->listLayoutsSrc->setSorting(1);	// from Qt3 QListView sorts by language


  if (!model.isEmpty())
  {
    QString m_name = rules->models()[model];
    widget->comboModel->setCurrentText(m_name);
  }
}


QString LayoutConfig::createOptionString()
{
  QString options;
  for (QDictIterator<char> it(rules->options()); it.current(); ++it)
  {
    QString option(it.currentKey());

    if (option.contains(':')) {

      QString optionKey = option.mid(0, option.find(':'));
      OptionListItem *item = optionGroups[optionKey];

      if( !item ) {
        kdDebug() << "WARNING: skipping empty group for " << it.currentKey()
          << endl;
        continue;
      }

      OptionListItem *child = item->findChildItem( option );

      if ( child ) {
        if ( child->state() == QCheckListItem::On ) {
          QString selectedName = child->optionName();
          if ( !selectedName.isEmpty() && selectedName != "none" ) {
            if (!options.isEmpty())
              options.append(',');
            options.append(selectedName);
          }
        }
      }
      else
        kdDebug() << "Empty option button for group " << it.currentKey() << endl;
    }
  }
  return options;
}

void LayoutConfig::save()
{
  KConfig *config = new KConfig("kxkbrc", false);
  config->setGroup("Layout");
  //  config->writeEntry("Rule", ruleCombo->currentText());

  QString model = lookupLocalized(rules->models(), widget->comboModel->currentText());
  config->writeEntry("Model", model);

  config->writeEntry("ResetOldOptions", widget->checkResetOld->isChecked());
  config->writeEntry("Options", createOptionString() );

  QStringList otherLayouts;
  QListViewItem *item = widget->listLayoutsDst->firstChild();
  if( item ) {
    QString layout = lookupLocalized(rules->layouts(), item->text(1));
    if( !layout.isEmpty() )
	config->writeEntry("Layout", layout);
    item = item->nextSibling();
  }
  else {
    widget->chkEnable->setChecked(false);
  }
  
  while (item) {
	QString layout = lookupLocalized(rules->layouts(), item->text(1));
	otherLayouts.append(layout);
      item = item->nextSibling();
  }
  config->writeEntry("Additional", otherLayouts);

  QStringList varList;
  for(QDictIterator<char> it(variants); it.current(); ++it )
  {
    QString var = it.currentKey();
//    if( var != layout && !otherLayouts.contains(var) ) // this layout was deleted
//      continue;

    var += "(";
    var += it.current();
    var += ")";
    varList.append( var );
  }
  config->writeEntry("Variants", varList);


  config->writeEntry("Use", widget->chkEnable->isChecked());
  config->writeEntry("ShowSingle", widget->chkShowSingle->isChecked());

  int modeId = widget->grpSwitching->id(widget->grpSwitching->selected());
  if( modeId < 1 || modeId > 2 )
    modeId = 0;

  config->writeEntry("SwitchMode", switchModes[modeId]);

  config->sync();

  delete config;

  kapp->kdeinitExec("kxkb");
}


void LayoutConfig::defaults()
{
  widget->chkEnable->setChecked(false);
  ruleChanged("xfree86");

  widget->comboModel->setCurrentText("pc104");
  //layoutCombo->setCurrentText("us");

  QListViewItem *item = widget->listLayoutsSrc->firstChild();
  while (item)
    {
      QCheckListItem *cli = dynamic_cast<QCheckListItem*>(item);
      cli->setOn(false);
      item = item->nextSibling();
    }
}

QString LayoutConfig::quickHelp() const
{
  return i18n("<h1>Keyboard Layout</h1> Here you can choose your keyboard layout and model."
    " The 'model' refers to the type of keyboard that is connected to your computer, while the keyboard"
    " layout defines \"which key does what\" and may be different for different countries.<p>"
    " In addition to the 'Primary Layout', which will be used as the default, you can specify additional"
    " layouts, which you can easily switch between using the KDE panel.");
}


extern "C"
{
  KCModule *create_keyboard_layout(QWidget *parent, const char *)
  {
    return new LayoutConfig(parent, "kcmlayout");
  };

  KCModule *create_keyboard(QWidget *parent, const char *)
  {
    return new KeyboardConfig(parent, "kcmlayout");
  }

  void init_keyboard()
  {
    KConfig *config = new KConfig("kcminputrc", true); // Read-only, no globals
    config->setGroup("Keyboard");

    XKeyboardState   kbd;
    XKeyboardControl kbdc;

    XGetKeyboardControl(kapp->getDisplay(), &kbd);
    bool key = config->readBoolEntry("KeyboardRepeating", true);
    kbdc.key_click_percent = config->readNumEntry("ClickVolume", kbd.key_click_percent);
    kbdc.auto_repeat_mode = (key ? AutoRepeatModeOn : AutoRepeatModeOff);

    XChangeKeyboardControl(kapp->getDisplay(),
                           KBKeyClickPercent | KBAutoRepeatMode,
                           &kbdc);

    if( key ) {
	int delay_ = config->readNumEntry("RepeatDelay", 250);
	int rate_ = config->readNumEntry("RepeatRate", 30);
	set_repeatrate(delay_, rate_);
    }


    int numlockState = config->readNumEntry( "NumLock", 2 );
    if( numlockState != 2 )
        numlockx_change_numlock_state( numlockState == 0 );

    delete config;

    config = new KConfig("kxkbrc", true, false);
    config->setGroup("Layout");
    if (config->readBoolEntry("Use", false) == true)
        kapp->startServiceByDesktopName("kxkb");
    delete config;
  }
}

#if 0// do not remove!
// These options are from XFree 4.1.0
 I18N_NOOP("Group Shift/Lock behavior");
 I18N_NOOP("R-Alt switches group while pressed");
 I18N_NOOP("Right Alt key changes group");
 I18N_NOOP("Caps Lock key changes group");
 I18N_NOOP("Menu key changes group");
 I18N_NOOP("Both Shift keys together change group");
 I18N_NOOP("Control+Shift changes group");
 I18N_NOOP("Alt+Control changes group");
 I18N_NOOP("Alt+Shift changes group");
 I18N_NOOP("Control Key Position");
 I18N_NOOP("Make Caps Lock an additional Control");
 I18N_NOOP("Swap Control and Caps Lock");
 I18N_NOOP("Control key at left of 'A'");
 I18N_NOOP("Control key at bottom left");
 I18N_NOOP("Use keyboard LED to show alternative group");
 I18N_NOOP("Num_Lock LED shows alternative group");
 I18N_NOOP("Caps_Lock LED shows alternative group");
 I18N_NOOP("Scroll_Lock LED shows alternative group");

//these seem to be new in XFree86 4.2.0
 I18N_NOOP("Left Win-key switches group while pressed");
 I18N_NOOP("Right Win-key switches group while pressed");
 I18N_NOOP("Both Win-keys switch group while pressed");
 I18N_NOOP("Left Win-key changes group");
 I18N_NOOP("Right Win-key changes group");
 I18N_NOOP("Third level choosers");
 I18N_NOOP("Press Right Control to choose 3rd level");
 I18N_NOOP("Press Menu key to choose 3rd level");
 I18N_NOOP("Press any of Win-keys to choose 3rd level");
 I18N_NOOP("Press Left Win-key to choose 3rd level");
 I18N_NOOP("Press Right Win-key to choose 3rd level");
 I18N_NOOP("CapsLock key behavior");
 I18N_NOOP("uses internal capitalization. Shift cancels Caps.");
 I18N_NOOP("uses internal capitalization. Shift doesn't cancel Cap$");
 I18N_NOOP("acts as Shift with locking. Shift cancels Caps.");
 I18N_NOOP("acts as Shift with locking. Shift doesn't cancel Caps.");
 I18N_NOOP("Alt/Win key behavior");
 I18N_NOOP("Add the standard behavior to Menu key.");
 I18N_NOOP("Alt and Meta on the Alt keys (default).");
 I18N_NOOP("Meta is mapped to the Win-keys.");
 I18N_NOOP("Meta is mapped to the left Win-key.");
 I18N_NOOP("Super is mapped to the Win-keys (default).");
 I18N_NOOP("Hyper is mapped to the Win-keys.");
 I18N_NOOP("Right Alt is Compose");
 I18N_NOOP("Right Win-key is Compose");
 I18N_NOOP("Menu is Compose");
#endif
